@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
  * @brief Class @ref Corrade::Containers::Array
  */
 
+#include <new>
 #include <type_traits>
 #include <utility>
 
@@ -36,10 +37,6 @@
 #include "Corrade/Containers/ArrayView.h"
 #include "Corrade/Containers/Tags.h"
 
-#ifdef CORRADE_BUILD_DEPRECATED
-#include "Corrade/Utility/Macros.h"
-#endif
-
 namespace Corrade { namespace Containers {
 
 namespace Implementation {
@@ -99,6 +96,8 @@ is possible to initialize the array in a different way using so-called *tags*:
     calls default constructor elsewhere.
 -   @ref Array(DirectInitT, std::size_t, Args&&... args) constructs all
     elements of the array using provided arguments.
+-   @ref Array(InPlaceInitT, std::initializer_list<T>) allocates unitialized
+    memory and then copy-constructs all elements from the initializer list
 -   @ref Array(NoInitT, std::size_t) does not initialize anything and you need
     to call the constructor on all elements manually using placement new,
     @ref std::uninitialized_copy() or similar. This is the dangerous option.
@@ -144,24 +143,6 @@ class Array {
         typedef T Type;     /**< @brief Element type */
         typedef D Deleter;  /**< @brief Deleter type */
 
-        #ifdef CORRADE_BUILD_DEPRECATED
-        /**
-         * @brief @copybrief Array(InPlaceInitT, std::initializer_list<T>)
-         * @deprecated Use @ref Array(InPlaceInitT, std::initializer_list<T>) instead.
-         */
-        template<class ...U> CORRADE_DEPRECATED("use Array(InPlaceInitT, std::initializer_list<T>) instead") static Array<T, D> from(U&&... values) {
-            return Array<T, D>{InPlaceInit, {T(std::forward<U>(values))...}};
-        }
-
-        /**
-         * @brief @copybrief Array(ValueInitT, std::size_t)
-         * @deprecated Use @ref Array(ValueInitT, std::size_t) instead.
-         */
-        CORRADE_DEPRECATED("use Array(ValueInitT, std::size_t) instead") static Array<T, D> zeroInitialized(std::size_t size) {
-            return Array<T>{ValueInit, size};
-        }
-        #endif
-
         /** @brief Conversion from nullptr */
         #ifdef DOXYGEN_GENERATING_OUTPUT
         /*implicit*/ Array(std::nullptr_t) noexcept:
@@ -282,6 +263,10 @@ class Array {
         explicit operator bool() const { return _data; }
         #endif
 
+        /* The following ArrayView conversion are *not* restricted to this&
+           because that would break uses like `consume(foo());`, where
+           `consume()` expects a view but `foo()` returns an owning array. */
+
         /**
          * @brief Convert to @ref ArrayView
          *
@@ -328,18 +313,10 @@ class Array {
            operators */
 
         /** @brief Conversion to array type */
-        /*implicit*/ operator T*()
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        &
-        #endif
-        { return _data; }
+        /*implicit*/ operator T*() & { return _data; }
 
         /** @overload */
-        /*implicit*/ operator const T*() const
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        &
-        #endif
-        { return _data; }
+        /*implicit*/ operator const T*() const & { return _data; }
 
         /** @brief Array data */
         T* data() { return _data; }
@@ -410,11 +387,11 @@ class Array {
         }
         /** @overload */
         ArrayView<T> slice(std::size_t begin, std::size_t end) {
-            return slice(_data + begin, _data + end);
+            return ArrayView<T>(*this).slice(begin, end);
         }
         /** @overload */
         ArrayView<const T> slice(std::size_t begin, std::size_t end) const {
-            return slice(_data + begin, _data + end);
+            return ArrayView<const T>(*this).slice(begin, end);
         }
 
         /**
@@ -432,11 +409,11 @@ class Array {
         }
         /** @overload */
         template<std::size_t size> StaticArrayView<size, T> slice(std::size_t begin) {
-            return slice<size>(_data + begin);
+            return ArrayView<T>(*this).template slice<size>(begin);
         }
         /** @overload */
         template<std::size_t size> StaticArrayView<size, const T> slice(std::size_t begin) const {
-            return slice<size>(_data + begin);
+            return ArrayView<const T>(*this).template slice<size>(begin);
         }
 
         /**
@@ -451,16 +428,22 @@ class Array {
         ArrayView<const T> prefix(const T* end) const {
             return ArrayView<const T>(*this).prefix(end);
         }
-        ArrayView<T> prefix(std::size_t end) { return prefix(_data + end); } /**< @overload */
-        ArrayView<const T> prefix(std::size_t end) const { return prefix(_data + end); } /**< @overload */
+        /** @overload */
+        ArrayView<T> prefix(std::size_t end) {
+            return ArrayView<T>(*this).prefix(end);
+        }
+        /** @overload */
+        ArrayView<const T> prefix(std::size_t end) const {
+            return ArrayView<const T>(*this).prefix(end);
+        }
 
         /** @overload */
         template<std::size_t viewSize> StaticArrayView<viewSize, T> prefix() {
-            return slice<viewSize>(_data);
+            return ArrayView<T>(*this).template prefix<viewSize>();
         }
         /** @overload */
         template<std::size_t viewSize> StaticArrayView<viewSize, const T> prefix() const {
-            return slice<viewSize>(_data);
+            return ArrayView<const T>(*this).template prefix<viewSize>();
         }
 
         /**
@@ -475,8 +458,14 @@ class Array {
         ArrayView<const T> suffix(const T* begin) const {
             return ArrayView<const T>(*this).suffix(begin);
         }
-        ArrayView<T> suffix(std::size_t begin) { return suffix(_data + begin); } /**< @overload */
-        ArrayView<const T> suffix(std::size_t begin) const { return suffix(_data + begin); } /**< @overload */
+        /** @overload */
+        ArrayView<T> suffix(std::size_t begin) {
+            return ArrayView<T>(*this).suffix(begin);
+        }
+        /** @overload */
+        ArrayView<const T> suffix(std::size_t begin) const {
+            return ArrayView<const T>(*this).suffix(begin);
+        }
 
         /**
          * @brief Release data storage
@@ -539,15 +528,6 @@ template<class T> std::size_t arraySize(const Array<T>& view) {
     return view.size();
 }
 
-#ifdef CORRADE_BUILD_DEPRECATED
-/** @brief @copybrief ArrayView
- * @deprecated Use @ref ArrayView.h and @ref ArrayView instead.
- */
-#ifndef CORRADE_MSVC2015_COMPATIBILITY /* Multiple definitions still broken */
-template<class T> using ArrayReference CORRADE_DEPRECATED_ALIAS("use ArrayView.h and ArrayView instead") = ArrayView<T>;
-#endif
-#endif
-
 template<class T, class D> inline Array<T, D>::Array(Array<T, D>&& other) noexcept: _data{other._data}, _size{other._size}, _deleter{other._deleter} {
     other._data = nullptr;
     other._size = 0;
