@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
  * @brief Class @ref Corrade::Containers::StaticArray
  */
 
+#include <new>
 #include <type_traits>
 #include <utility>
 
@@ -36,10 +37,6 @@
 #include "Corrade/Containers/ArrayView.h"
 #include "Corrade/Containers/Tags.h"
 
-#ifdef CORRADE_BUILD_DEPRECATED
-#include "Corrade/Utility/Macros.h"
-#endif
-
 namespace Corrade { namespace Containers {
 
 /**
@@ -52,16 +49,7 @@ as more featureful alternative to plain C arrays or @ref std::array.
 
 Usage example:
 
-@code{.cpp}
-// Create default-initialized array with 5 integers and set them to some value
-Containers::StaticArray<5, int> a;
-int b = 0;
-for(auto& i: a) i = b++; // a = {0, 1, 2, 3, 4}
-
-// Create array from given values
-Containers::StaticArray<4, int> b{3, 18, -157, 0};
-b[3] = 25; // b = {3, 18, -157, 25}
-@endcode
+@snippet Containers.cpp StaticArray-usage
 
 @section Containers-StaticArray-initialization Array initialization
 
@@ -73,7 +61,7 @@ is possible to initialize the array in a different way using so-called *tags*:
     constructor (useful when you want to make the choice appear explicit).
 -   @ref StaticArray(InPlaceInitT, Args&&... args) is equivalent to the
     implicit parameteric constructor (again useful when you want to make the
-    choice appear explicit).
+    choice appear explicit). Same as @ref StaticArray(Args&&... args).
 -   @ref StaticArray(ValueInitT) zero-initializes trivial types and calls
     default constructor elsewhere.
 -   @ref StaticArray(DirectInitT, Args&&... args) constructs every element of
@@ -84,32 +72,7 @@ is possible to initialize the array in a different way using so-called *tags*:
 
 Example:
 
-@code{.cpp}
-// These two are equivalent
-Containers::StaticArray<5, int> a1;
-Containers::StaticArray<5, int> a2{Containers::DefaultInit};
-
-// Array of 100 zeros
-Containers::StaticArray<100, int> b{Containers::ValueInit};
-
-// Array of 4 values initialized in-place (these two are equivalent)
-Containers::StaticArray<4, int> c1{3, 18, -157, 0};
-Containers::StaticArray<4, int> c2{Containers::InPlaceInit, 3, 18, -157, 0};
-
-// Array of type with no default constructor
-struct Vec3 {
-    Vec3(float, float, float);
-};
-Containers::StaticArray<5, Vec3> d{Containers::DirectInit, 5.2f, 0.4f, 1.0f};
-
-// Manual construction of each element
-struct Foo {
-    Foo(int index);
-};
-Containers::StaticArray<5, Foo> e{Containers::NoInit};
-int index = 0;
-for(Foo& f: e) new(&f) Foo(index++);
-@endcode
+@snippet Containers.cpp StaticArray-initialization
 
 @see @ref arrayCast(StaticArray<size, T>&)
 */
@@ -171,7 +134,8 @@ template<std::size_t size_, class T> class StaticArray {
         /**
          * @brief Construct in-place-initialized array
          *
-         * The arguments are forwarded to the array constructor.
+         * The arguments are forwarded to the array constructor. Same as
+         * @ref StaticArray(Args&&... args).
          * @see @ref StaticArray(DirectInitT, Args&&... args)
          */
         template<class ...Args> explicit StaticArray(InPlaceInitT, Args&&... args): _data{std::forward<Args>(args)...} {
@@ -193,9 +157,9 @@ template<std::size_t size_, class T> class StaticArray {
          * @see @ref StaticArray(DirectInitT, Args&&... args)
          */
         #ifdef DOXYGEN_GENERATING_OUTPUT
-        template<class ...Args> explicit StaticArray(Args&&... args);
+        template<class ...Args> /*implicit*/ StaticArray(Args&&... args);
         #else
-        template<class First, class ...Next> explicit StaticArray(First&& first, Next&&... next): StaticArray{InPlaceInit, std::forward<First>(first), std::forward<Next>(next)...} {}
+        template<class First, class ...Next> /*implicit*/ StaticArray(First&& first, Next&&... next): StaticArray{InPlaceInit, std::forward<First>(first), std::forward<Next>(next)...} {}
         #endif
 
         /** @brief Copying is not allowed */
@@ -212,6 +176,10 @@ template<std::size_t size_, class T> class StaticArray {
         /** @brief Moving is not allowed */
         StaticArray<size_, T>& operator=(StaticArray<size_, T>&&) = delete;
 
+        /* The following ArrayView conversion are *not* restricted to this&
+           because that would break uses like `consume(foo());`, where
+           `consume()` expects a view but `foo()` returns an owning array. */
+
         /**
          * @brief Convert to @ref ArrayView
          *
@@ -293,18 +261,10 @@ template<std::size_t size_, class T> class StaticArray {
            operators */
 
         /** @brief Conversion to array type */
-        /*implicit*/ operator T*()
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        &
-        #endif
-        { return _data; }
+        /*implicit*/ operator T*() & { return _data; }
 
         /** @overload */
-        /*implicit*/ operator const T*() const
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        &
-        #endif
-        { return _data; }
+        /*implicit*/ operator const T*() const & { return _data; }
 
         /** @brief Array data */
         T* data() { return _data; }
@@ -324,18 +284,42 @@ template<std::size_t size_, class T> class StaticArray {
          */
         constexpr bool empty() const { return !size_; }
 
-        /** @brief Pointer to first element */
+        /**
+         * @brief Pointer to first element
+         *
+         * @see @ref front()
+         */
         T* begin() { return _data; }
         const T* begin() const { return _data; }            /**< @overload */
         const T* cbegin() const { return _data; }           /**< @overload */
 
-        /** @brief Pointer to (one item after) last element */
+        /**
+         * @brief Pointer to (one item after) last element
+         *
+         * @see @ref back()
+         */
         T* end() { return _data + size_; }
         const T* end() const { return _data + size_; }      /**< @overload */
         const T* cend() const { return _data + size_; }     /**< @overload */
 
         /**
-         * @brief Reference to array slice
+         * @brief First element
+         *
+         * @see @ref begin()
+         */
+        T& front() { return _data[0]; }
+        const T& front() const { return _data[0]; }         /**< @overload */
+
+        /**
+         * @brief Last element
+         *
+         * @see @ref end()
+         */
+        T& back() { return _data[size_ - 1]; }
+        const T& back() const { return _data[size_ - 1]; }  /**< @overload */
+
+        /**
+         * @brief Array slice
          *
          * Equivalent to @ref ArrayView::slice().
          */
@@ -348,11 +332,11 @@ template<std::size_t size_, class T> class StaticArray {
         }
         /** @overload */
         ArrayView<T> slice(std::size_t begin, std::size_t end) {
-            return slice(_data + begin, _data + end);
+            return ArrayView<T>(*this).slice(begin, end);
         }
         /** @overload */
         ArrayView<const T> slice(std::size_t begin, std::size_t end) const {
-            return slice(_data + begin, _data + end);
+            return ArrayView<const T>(*this).slice(begin, end);
         }
 
         /**
@@ -370,11 +354,11 @@ template<std::size_t size_, class T> class StaticArray {
         }
         /** @overload */
         template<std::size_t viewSize> StaticArrayView<viewSize, T> slice(std::size_t begin) {
-            return slice<viewSize>(_data + begin);
+            return ArrayView<T>(*this).template slice<viewSize>(begin);
         }
         /** @overload */
         template<std::size_t viewSize> StaticArrayView<viewSize, const T> slice(std::size_t begin) const {
-            return slice<viewSize>(_data + begin);
+            return ArrayView<const T>(*this).template slice<viewSize>(begin);
         }
 
         /**
@@ -389,8 +373,23 @@ template<std::size_t size_, class T> class StaticArray {
         ArrayView<const T> prefix(const T* end) const {
             return ArrayView<const T>(*this).prefix(end);
         }
-        ArrayView<T> prefix(std::size_t end) { return prefix(_data + end); } /**< @overload */
-        ArrayView<const T> prefix(std::size_t end) const { return prefix(_data + end); } /**< @overload */
+        /** @overload */
+        ArrayView<T> prefix(std::size_t end) {
+            return ArrayView<T>(*this).prefix(end);
+        }
+        /** @overload */
+        ArrayView<const T> prefix(std::size_t end) const {
+            return ArrayView<const T>(*this).prefix(end);
+        }
+
+        /**
+         * @brief Static array prefix
+         *
+         * Expects (at compile-time) that @p viewSize is not larger than
+         * @ref Size.
+         */
+        template<std::size_t viewSize> StaticArrayView<viewSize, T> prefix();
+        template<std::size_t viewSize> StaticArrayView<viewSize, const T> prefix() const; /**< @overload */
 
         /**
          * @brief Array suffix
@@ -404,8 +403,14 @@ template<std::size_t size_, class T> class StaticArray {
         ArrayView<const T> suffix(const T* begin) const {
             return ArrayView<const T>(*this).suffix(begin);
         }
-        ArrayView<T> suffix(std::size_t begin) { return suffix(_data + begin); } /**< @overload */
-        ArrayView<const T> suffix(std::size_t begin) const { return suffix(_data + begin); } /**< @overload */
+        /** @overload */
+        ArrayView<T> suffix(std::size_t begin) {
+            return ArrayView<T>(*this).suffix(begin);
+        }
+        /** @overload */
+        ArrayView<const T> suffix(std::size_t begin) const {
+            return ArrayView<const T>(*this).suffix(begin);
+        }
 
     private:
         explicit StaticArray(DefaultInitT, std::true_type) {}
@@ -431,12 +436,7 @@ template<std::size_t size_, class T> class StaticArray {
 Convenience alternative to calling @ref StaticArray::operator ArrayView<U>()
 explicitly. The following two lines are equivalent:
 
-@code{.cpp}
-Containers::StaticArray<5, std::uint32_t> data;
-
-Containers::ArrayView<std::uint32_t> a{data};
-auto b = Containers::arrayView(data);
-@endcode
+@snippet Containers.cpp StaticArray-arrayView
 */
 template<std::size_t size, class T> constexpr ArrayView<T> arrayView(StaticArray<size, T>& array) {
     return ArrayView<T>{array};
@@ -448,12 +448,7 @@ template<std::size_t size, class T> constexpr ArrayView<T> arrayView(StaticArray
 Convenience alternative to calling @ref StaticArray::operator ArrayView<U>()
 explicitly. The following two lines are equivalent:
 
-@code{.cpp}
-const Containers::StaticArray<5, std::uint32_t> data;
-
-Containers::ArrayView<const std::uint32_t> a{data};
-auto b = Containers::arrayView(data);
-@endcode
+@snippet Containers.cpp StaticArray-arrayView-const
 */
 template<std::size_t size, class T> constexpr ArrayView<const T> arrayView(const StaticArray<size, T>& array) {
     return ArrayView<const T>{array};
@@ -465,12 +460,7 @@ template<std::size_t size, class T> constexpr ArrayView<const T> arrayView(const
 Convenience alternative to calling @cpp StaticArray::operator StaticArrayView<size_, U>() @ce
 explicitly. The following two lines are equivalent:
 
-@code{.cpp}
-Containers::StaticArray<5, std::uint32_t> data;
-
-Containers::StaticArrayView<5, std::uint32_t> a{data};
-auto b = Containers::staticArrayView(data);
-@endcode
+@snippet Containers.cpp StaticArray-staticArrayView
 
 @todoc Make it a real reference once Doxygen is sane
 */
@@ -484,12 +474,7 @@ template<std::size_t size, class T> constexpr StaticArrayView<size, T> staticArr
 Convenience alternative to calling @cpp StaticArray::operator StaticArrayView<size_, U>() @ce
 explicitly. The following two lines are equivalent:
 
-@code{.cpp}
-const Containers::StaticArray<5, std::uint32_t> data;
-
-Containers::StaticArrayView<5, const std::uint32_t> a{data};
-auto b = Containers::staticArrayView(data);
-@endcode
+@snippet Containers.cpp StaticArray-staticArrayView-const
 
 @todoc Make it a real reference once Doxygen is sane
 */
@@ -531,6 +516,16 @@ template<std::size_t size_, class T> StaticArray<size_, T>::~StaticArray() {
     for(T& i: _data) i.~T();
 }
 
+template<std::size_t size_, class T> template<std::size_t viewSize> StaticArrayView<viewSize, T> StaticArray<size_, T>::prefix() {
+    static_assert(viewSize <= size_, "prefix size too large");
+    return StaticArrayView<viewSize, T>{_data};
+}
+
+template<std::size_t size_, class T> template<std::size_t viewSize> StaticArrayView<viewSize, const T> StaticArray<size_, T>::prefix() const {
+    static_assert(viewSize <= size_, "prefix size too large");
+    return StaticArrayView<viewSize, const T>{_data};
+}
+
 }}
 
 #endif
