@@ -3,8 +3,8 @@
 /*
     This file is part of Corrade.
 
-    Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -26,26 +26,28 @@
 */
 
 /** @file
- * @brief Class @ref Corrade::PluginManager::AbstractManager
+ * @brief Class @ref Corrade::PluginManager::AbstractManager, macro @ref CORRADE_PLUGIN_VERSION, @ref CORRADE_PLUGIN_REGISTER()
  */
 
-#include <iosfwd>
 #include <map>
 #include <string>
 #include <vector>
 
 #include "Corrade/Containers/EnumSet.h"
-#include "Corrade/PluginManager/PluginMetadata.h"
 #include "Corrade/PluginManager/PluginManager.h"
 #include "Corrade/PluginManager/visibility.h"
-#include "Corrade/Utility/Configuration.h"
-#include "Corrade/Utility/Debug.h"
 #include "Corrade/Utility/Resource.h"
 
 #ifdef CORRADE_TARGET_WINDOWS
 /* I didn't find a better way to circumvent the need for including windows.h */
 struct HINSTANCE__;
 typedef struct HINSTANCE__* HMODULE;
+
+/* combaseapi.h in Windows does this insane thing. Can be worked around by
+   defining WIN32_LEAN_AND_MEAN but not everybody does that. */
+#ifdef interface
+#undef interface
+#endif
 #endif
 
 namespace Corrade { namespace PluginManager {
@@ -63,54 +65,44 @@ enum class LoadState: unsigned short {
      */
     NotFound = 1 << 0,
 
-    #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
+    #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
     /**
      * The plugin is build with different version of plugin manager and cannot
      * be loaded. Returned by @ref AbstractManager::load().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     WrongPluginVersion = 1 << 1,
 
     /**
      * The plugin uses different interface than the interface used by plugin
      * manager and cannot be loaded. Returned by @ref AbstractManager::load().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     WrongInterfaceVersion = 1 << 2,
 
     /**
      * The plugin doesn't have any metadata file or the metadata file contains
      * errors. Returned by @ref AbstractManager::load().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     WrongMetadataFile = 1 << 3,
 
     /**
      * The plugin depends on another plugin, which cannot be loaded (e.g. not
      * found or wrong version). Returned by @ref AbstractManager::load().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     UnresolvedDependency = 1 << 4,
 
     /**
      * The plugin failed to load for other reason (e.g. linking failure).
      * Returned by @ref AbstractManager::load().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     LoadFailed = 1 << 5,
     #endif
@@ -124,62 +116,55 @@ enum class LoadState: unsigned short {
     /**
      * The plugin is successfully loaded. Returned by @ref AbstractManager::loadState()
      * and @ref AbstractManager::load(). The value includes value of @ref LoadState::Static,
-     * see @ref LoadStates for more information. In @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     * @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and @ref CORRADE_TARGET_WINDOWS_RT "Windows RT"
-     * the value is equivalent to @ref LoadState::Static.
+     * see @ref LoadStates for more information. In @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten",
+     * @ref CORRADE_TARGET_WINDOWS_RT "Windows RT", @ref CORRADE_TARGET_IOS "iOS"
+     * and @ref CORRADE_TARGET_ANDROID the value is equivalent to
+     * @ref LoadState::Static.
      */
-    #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
+    #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
     Loaded = (1 << 7) | LoadState::Static,
     #else
     Loaded = LoadState::Static,
     #endif
 
-    #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
+    #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
     /**
      * The plugin is not loaded. Plugin can be unloaded only if is dynamic and
      * is not required by any other plugin. Returned by
      * @ref AbstractManager::loadState() and @ref AbstractManager::load().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     NotLoaded = 1 << 8,
 
     /**
      * The plugin failed to unload. Returned by @ref AbstractManager::unload().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     UnloadFailed = 1 << 9,
 
     /**
      * The plugin cannot be unloaded because another plugin is depending on it.
      * Unload that plugin first and try again. Returned by @ref AbstractManager::unload().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     Required = 1 << 10,
-    #endif
 
-    #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
     /**
-     * The plugin has active instance and cannot be unloaded. Destroy all
-     * instances and try again. Returned by @ref AbstractManager::unload().
-     * @partialsupport Only static plugins are supported in
-     *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-     *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-     *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+     * @ref AbstractManager::unload() returns this if the plugin has an active
+     * instance and cannot be unloaded. Destroy all instances and try again.
+     * @ref AbstractManager::load() returns this if loading a file path
+     * directly and a plugin with the same name already exists.
+     * @partialsupport Not available on platforms without
+     *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
      */
     Used = 1 << 11
     #endif
 };
 
-/** @debugoperatorenum{Corrade::PluginManager::LoadState} */
+/** @debugoperatorenum{LoadState} */
 CORRADE_PLUGINMANAGER_EXPORT Utility::Debug& operator<<(Utility::Debug& debug, PluginManager::LoadState value);
 
 /**
@@ -187,15 +172,13 @@ CORRADE_PLUGINMANAGER_EXPORT Utility::Debug& operator<<(Utility::Debug& debug, P
 
 Useful when checking whether @ref LoadState in in given set of values, for
 example:
-@code
-if(loadState & (LoadState::WrongPluginVersion|LoadState::WrongInterfaceVersion)) {
-    // ...
-}
-@endcode
 
-Note that @ref LoadState::Loaded includes value of @ref LoadState::Static, so
-you can use `loadState & LoadState::Loaded` instead of much more verbose
-`state & (LoadState::Loaded|LoadState::Static)`.
+@snippet PluginManager.cpp LoadStates
+
+Note that @ref LoadState::Loaded includes the value of @ref LoadState::Static,
+so you can use @cpp loadState & PluginManager::LoadState::Loaded @ce instead of
+much more verbose
+@cpp loadState & (PluginManager::LoadState::Loaded|PluginManager::LoadState::Static) @ce.
 @see @ref AbstractManager::loadState(), @ref AbstractManager::load(),
     @ref AbstractManager::unload()
 */
@@ -204,9 +187,9 @@ typedef Containers::EnumSet<LoadState> LoadStates;
 CORRADE_ENUMSET_OPERATORS(LoadStates)
 
 /**
-@brief Non-templated base class of @ref Manager
+@brief Non-templated base for plugin managers
 
-See also @ref plugin-management.
+See @ref Manager and @ref plugin-management for more information.
  */
 class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
     friend AbstractPlugin;
@@ -216,7 +199,7 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
         static const int Version;
 
         #ifndef DOXYGEN_GENERATING_OUTPUT
-        typedef void* (*Instancer)(AbstractManager&, std::string);
+        typedef void* (*Instancer)(AbstractManager&, const std::string&);
         static void importStaticPlugin(std::string plugin, int _version, std::string interface, Instancer instancer, void(*initializer)(), void(*finalizer)());
         #endif
 
@@ -240,14 +223,12 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
          */
         std::string pluginInterface() const;
 
-        #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
+        #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
         /**
          * @brief Plugin directory
          *
-         * @partialsupport Only static plugins are supported in
-         *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib,
-         *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-         *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+         * @partialsupport Not available on platforms without
+         *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
          */
         std::string pluginDirectory() const;
 
@@ -255,37 +236,65 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
          * @brief Set another plugin directory
          *
          * Keeps loaded plugins untouched, removes unloaded plugins which are
-         * not existing anymore and adds newly found plugins.
-         * @partialsupport Only static plugins are supported in
-         *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-         *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-         *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+         * not existing anymore and adds newly found plugins. The directory is
+         * expected to be in UTF-8.
+         * @partialsupport Not available on platforms without
+         *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
          */
         void setPluginDirectory(std::string directory);
 
         /**
          * @brief Reload plugin directory
          *
-         * Convenience equivalent to `setPluginDirectory(pluginDirectory())`.
-         * @partialsupport Only static plugins are supported in
-         *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-         *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-         *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+         * Convenience equivalent to @cpp setPluginDirectory(pluginDirectory()) @ce.
+         * @partialsupport Not available on platforms without
+         *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support".
          */
         void reloadPluginDirectory();
         #endif
 
-        /** @brief List of all available plugin names */
+        /**
+         * @brief Set preferred plugins for given alias
+         *
+         * By default, if more than one plugin provides given @p alias, one of
+         * them is arbitrarily chosen. With this function it's possible to
+         * control the behavior. For given @p alias the function goes through
+         * the list in @p plugins and uses the first plugin that is available.
+         * The @p alias is expected to exist and be defined by plugins in
+         * @p plugins. If none of @p plugins is available, nothing is done.
+         *
+         * Note that after calling @ref setPluginDirectory() or @ref reloadPluginDirectory()
+         * this preference gets reset and you may need to call this function
+         * again.
+         */
+        void setPreferredPlugins(const std::string& alias, std::initializer_list<std::string> plugins);
+
+        /**
+         * @brief List of all available plugin names
+         *
+         * Returns a list of names that correspond to concrete unique static or
+         * dynamic plugins.
+         * @see @ref aliasList()
+         */
         std::vector<std::string> pluginList() const;
 
+        /**
+         * @brief List of all available alias names
+         *
+         * In addition to everything returned by @ref pluginList() contains
+         * also all plugin aliases.
+         */
+        std::vector<std::string> aliasList() const;
+
         /**
          * @brief Plugin metadata
          *
-         * Returns pointer to plugin metadata or `nullptr`, if given plugin is
-         * not found.
+         * Returns pointer to plugin metadata or @cpp nullptr @ce, if given
+         * plugin is not found.
          * @see @ref AbstractPlugin::metadata()
          */
-        const PluginMetadata* metadata(const std::string& plugin) const;
+        PluginMetadata* metadata(const std::string& plugin);
+        const PluginMetadata* metadata(const std::string& plugin) const; /**< @overload */
 
         /**
          * @brief Load state of a plugin
@@ -311,12 +320,25 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
          * If the plugin has any dependencies, they are recursively processed
          * before loading given plugin.
          *
+         * If @p plugin is a plugin file path (i.e., ending with a
+         * platform-specific extension such as `.so` or `.dll`), it's loaded
+         * from given location and, if the loading succeeds, its basename
+         * (without extension) is exposed as an available plugin name. It's
+         * expected that a plugin with the same name is not already loaded. The
+         * plugin will reside in the plugin list as long as it's loaded or,
+         * after calling @ref unload() on it, until next call to
+         * @ref setPluginDirectory() or @ref reloadPluginDirectory().
+         *
+         * @note If passing a file path, the implementation expects forward
+         *      slashes as directory separators. Use @ref Utility::Directory::fromNativeSeparators()
+         *      to convert from platform-specific format.
+         *
          * @see @ref unload(), @ref loadState(), @ref Manager::instance(),
          *      @ref Manager::loadAndInstantiate()
-         * @partialsupport Only static plugins are supported in
-         *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-         *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-         *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+         * @partialsupport On platforms without
+         *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support"
+         *      returns always either @ref LoadState::Static or
+         *      @ref LoadState::NotFound.
          */
         LoadState load(const std::string& plugin);
 
@@ -325,15 +347,15 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
          *
          * Returns @ref LoadState::NotLoaded if the plugin is not loaded or
          * unloading succeeded. For static plugins always returns
-         * @ref LoadState::Static. On failure returns
+         * @ref LoadState::Static. On failure returns @ref LoadState::NotFound,
          * @ref LoadState::UnloadFailed, @ref LoadState::Required or
          * @ref LoadState::Used.
          *
          * @see @ref load(), @ref loadState()
-         * @partialsupport Only static plugins are supported in
-         *      @ref CORRADE_TARGET_NACL_NEWLIB "NaCl newlib",
-         *      @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" and
-         *      @ref CORRADE_TARGET_WINDOWS_RT "Windows RT".
+         * @partialsupport On platforms without
+         *      @ref CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT "dynamic plugin support"
+         *      returns always either @ref LoadState::Static or
+         *      @ref LoadState::NotFound.
          */
         LoadState unload(const std::string& plugin);
 
@@ -352,70 +374,17 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
     #else
     protected:
     #endif
-        struct StaticPlugin {
-            std::string plugin;
-            std::string interface;
-            Instancer instancer;
-            void(*initializer)();
-            void(*finalizer)();
-        };
+        struct CORRADE_PLUGINMANAGER_LOCAL StaticPlugin;
+        struct CORRADE_PLUGINMANAGER_LOCAL Plugin;
+        struct CORRADE_PLUGINMANAGER_LOCAL GlobalPluginStorage;
 
-        struct Plugin {
-            #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
-            LoadState loadState;
-            #else
-            const LoadState loadState; /* Always LoadState::Static */
-            #endif
-            Utility::Configuration configuration;
-            PluginMetadata metadata;
+        #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
+        explicit AbstractManager(std::string pluginInterface, const std::vector<std::string>& pluginSearchPaths, std::string pluginDirectory);
+        #else
+        explicit AbstractManager(std::string pluginInterface);
+        #endif
 
-            /* If set to nullptr, the plugin has not any associated plugin
-               manager and cannot be loaded. */
-            AbstractManager* manager;
-
-            Instancer instancer;
-
-            #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN)
-            union {
-                /* For static plugins */
-                StaticPlugin* staticPlugin;
-
-                /* For dynamic plugins */
-                #ifndef CORRADE_TARGET_WINDOWS
-                void* module;
-                #else
-                HMODULE module;
-                #endif
-            };
-            #else
-            StaticPlugin* staticPlugin;
-            #endif
-
-            #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
-            /* Constructor for dynamic plugins */
-            explicit Plugin(std::string name, const std::string& metadata, AbstractManager* manager);
-            #endif
-
-            /* Constructor for static plugins */
-            explicit Plugin(std::string name, std::istream& metadata, StaticPlugin* staticPlugin);
-
-            /* Ensure that we don't delete staticPlugin twice */
-            Plugin(const Plugin&) = delete;
-            Plugin(Plugin&&) = delete;
-            Plugin& operator=(const Plugin&) = delete;
-            Plugin& operator=(Plugin&&) = delete;
-
-            ~Plugin();
-        };
-
-        struct GlobalPluginStorage {
-            std::map<std::string, Plugin*> plugins;
-            std::map<std::string, Plugin&> aliases;
-        };
-
-        explicit AbstractManager(std::string pluginInterface, std::string pluginDirectory);
-
-        #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
+        #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
         std::string _pluginDirectory;
         #endif
 
@@ -423,11 +392,12 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
            and fills it with entries from staticPlugins(). The reference is
            then in constructor stored in _plugins variable to avoid at least
            some issues with duplicated static variables on static builds. */
-        static GlobalPluginStorage& initializeGlobalPluginStorage();
+        static CORRADE_PLUGINMANAGER_LOCAL GlobalPluginStorage& initializeGlobalPluginStorage();
 
         GlobalPluginStorage& _plugins;
 
-        void* instanceInternal(const std::string& plugin);
+        Containers::Pointer<AbstractPlugin> instantiateInternal(const std::string& plugin);
+        Containers::Pointer<AbstractPlugin> loadAndInstantiateInternal(const std::string& plugin);
 
     private:
         /* Temporary storage of all information needed to import static plugins.
@@ -442,21 +412,22 @@ class CORRADE_PLUGINMANAGER_EXPORT AbstractManager {
            be uninitalized when accessed from CORRADE_PLUGIN_REGISTER(). */
         CORRADE_PLUGINMANAGER_LOCAL static std::vector<StaticPlugin*>*& staticPlugins();
 
-        CORRADE_PLUGINMANAGER_LOCAL void registerInstance(std::string plugin, AbstractPlugin& instance, const PluginMetadata*& metadata);
+        CORRADE_PLUGINMANAGER_LOCAL void registerDynamicPlugin(const std::string& name, Plugin* plugin);
+
+        CORRADE_PLUGINMANAGER_LOCAL void registerInstance(const std::string& plugin, AbstractPlugin& instance, const PluginMetadata*& metadata);
         CORRADE_PLUGINMANAGER_LOCAL void unregisterInstance(const std::string& plugin, AbstractPlugin& instance);
 
-        CORRADE_PLUGINMANAGER_LOCAL Plugin* findWithAlias(const std::string& plugin);
-        CORRADE_PLUGINMANAGER_LOCAL const Plugin* findWithAlias(const std::string& plugin) const;
-
-        #if !defined(CORRADE_TARGET_NACL_NEWLIB) && !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_WINDOWS_RT)
+        #ifndef CORRADE_PLUGINMANAGER_NO_DYNAMIC_PLUGIN_SUPPORT
         CORRADE_PLUGINMANAGER_LOCAL LoadState loadInternal(Plugin& plugin);
+        CORRADE_PLUGINMANAGER_LOCAL LoadState loadInternal(Plugin& plugin, const std::string& filename);
         CORRADE_PLUGINMANAGER_LOCAL LoadState unloadInternal(Plugin& plugin);
         CORRADE_PLUGINMANAGER_LOCAL LoadState unloadRecursive(const std::string& plugin);
         CORRADE_PLUGINMANAGER_LOCAL LoadState unloadRecursiveInternal(Plugin& plugin);
         #endif
 
         std::string _pluginInterface;
-        std::map<std::string, std::vector<AbstractPlugin*> > instances;
+        std::map<std::string, Plugin&> _aliases;
+        std::map<std::string, std::vector<AbstractPlugin*>> _instances;
 };
 
 /** @hideinitializer
@@ -474,7 +445,10 @@ loaded at startup, so you need to load them explicitly by calling
 @ref CORRADE_PLUGIN_IMPORT() at the beginning of `main()` function. You can
 also wrap these macro calls into another function (which will then be compiled
 into dynamic library or main executable) and use @ref CORRADE_AUTOMATIC_INITIALIZER()
-macro for automatic call.
+macro for automatic call:
+
+@snippet PluginManager.cpp CORRADE_PLUGIN_IMPORT
+
 @attention This macro should be called outside of any namespace. If you are
     running into linker errors with `pluginImporter_*`, this could be the
     problem. See @ref CORRADE_RESOURCE_INITIALIZE() documentation for more
@@ -485,6 +459,69 @@ macro for automatic call.
     pluginImporter_##name();                                                \
     CORRADE_RESOURCE_INITIALIZE(name)
 
+/** @brief Plugin version */
+#define CORRADE_PLUGIN_VERSION 5
+
+/** @hideinitializer
+@brief Register static or dynamic lugin
+@param name          Name of static plugin (equivalent of dynamic plugin
+     filename)
+@param className     Plugin class name
+@param interface     Interface name (the same as is defined by the
+    @ref Corrade::PluginManager::AbstractPlugin::pluginInterface() "pluginInterface()"
+    member of given plugin class)
+
+If the plugin is built as **static** (using CMake command
+@ref corrade-cmake-add-static-plugin "corrade_add_static_plugin()"), registers
+it, so it will be loaded automatically when PluginManager instance with
+corresponding interface is created. When building as static plugin,
+`CORRADE_STATIC_PLUGIN` preprocessor directive is defined.
+
+If the plugin is built as **dynamic** (using CMake command
+@ref corrade-cmake-add-plugin "corrade_add_plugin()"), registers it, so it can
+be dynamically loaded via @ref Corrade::PluginManager::Manager by supplying a
+name of the plugin. When building as dynamic plugin, `CORRADE_DYNAMIC_PLUGIN`
+preprocessor directive is defined.
+
+If the plugin is built as dynamic or static **library or executable** (not as
+plugin, using e.g. CMake command @cmake add_library() @ce /
+@cmake add_executable() @ce), this macro won't do anything to prevent linker
+issues when linking more plugins together. No plugin-related preprocessor
+directive is defined.
+
+See @ref plugin-management for more information about plugin compilation.
+
+@attention This macro should be called outside of any namespace. If you are
+    running into linker errors with `pluginImporter_`, this could be the
+    problem.
+*/
+#ifdef CORRADE_STATIC_PLUGIN
+#define CORRADE_PLUGIN_REGISTER(name, className, interface)                 \
+    inline void* pluginInstancer_##name(Corrade::PluginManager::AbstractManager& manager, const std::string& plugin) \
+        { return new className(manager, plugin); }                          \
+    int pluginImporter_##name();                                            \
+    int pluginImporter_##name() {                                           \
+        Corrade::PluginManager::AbstractManager::importStaticPlugin(#name, CORRADE_PLUGIN_VERSION, interface, pluginInstancer_##name, className::initialize, className::finalize); return 1; \
+    }
+#elif defined(CORRADE_DYNAMIC_PLUGIN)
+#define CORRADE_PLUGIN_REGISTER(name, className, interface)                 \
+    extern "C" CORRADE_PLUGIN_EXPORT int pluginVersion();                   \
+    extern "C" CORRADE_PLUGIN_EXPORT int pluginVersion() { return CORRADE_PLUGIN_VERSION; } \
+    extern "C" CORRADE_PLUGIN_EXPORT void* pluginInstancer(Corrade::PluginManager::AbstractManager& manager, const std::string& plugin); \
+    extern "C" CORRADE_PLUGIN_EXPORT void* pluginInstancer(Corrade::PluginManager::AbstractManager& manager, const std::string& plugin) \
+        { return new className{manager, plugin}; }                          \
+    extern "C" CORRADE_PLUGIN_EXPORT void pluginInitializer();              \
+    extern "C" CORRADE_PLUGIN_EXPORT void pluginInitializer()               \
+        { className::initialize(); }                                        \
+    extern "C" CORRADE_PLUGIN_EXPORT void pluginFinalizer();                \
+    extern "C" CORRADE_PLUGIN_EXPORT void pluginFinalizer()                 \
+        { className::finalize(); }                                          \
+    extern "C" CORRADE_PLUGIN_EXPORT const char* pluginInterface();         \
+    extern "C" CORRADE_PLUGIN_EXPORT const char* pluginInterface() { return interface; }
+#else
+#define CORRADE_PLUGIN_REGISTER(name, className, interface)
+#endif
+
 }}
 
 #endif
