@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 */
 
 /** @file
- * @brief Class @ref Corrade::Utility::Debug, @ref Corrade::Utility::Warning, @ref Corrade::Utility::Error
+ * @brief Class @ref Corrade::Utility::Debug, @ref Corrade::Utility::Warning, @ref Corrade::Utility::Error, @ref Corrade::Utility::Fatal
  */
 
 #include <iosfwd>
@@ -139,34 +139,7 @@ class CORRADE_UTILITY_EXPORT Debug {
          */
         typedef Containers::EnumSet<Flag> Flags;
 
-        #ifdef CORRADE_BUILD_DEPRECATED
-        /**
-         * @brief Debug output without newline at the end
-         * @deprecated Use @ref Debug(Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Debug(Flags) instead") static Debug noNewlineAtTheEnd() {
-            return Debug{Flag::NoNewlineAtTheEnd};
-        }
-
-        /**
-         * @brief Debug output without newline at the end
-         * @deprecated Use @ref Debug(std::ostream*, Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Debug(std::ostream*, Flags) instead") static Debug noNewlineAtTheEnd(std::ostream* output) {
-            return Debug{output, Flag::NoNewlineAtTheEnd};
-        }
-        #endif
-
-        #ifdef CORRADE_BUILD_DEPRECATED
-        /**
-         * @brief Set output for instances in this scope
-         * @deprecated Use @ref Debug(std::ostream*, Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Debug(std::ostream*, Flags) instead") static void setOutput(std::ostream* output);
-        #endif
-
         /** @{ @name Output modifiers
-         *
          * See @ref Utility-Debug-modifiers for more information.
          */
 
@@ -395,7 +368,7 @@ class CORRADE_UTILITY_EXPORT Debug {
         /**
          * @brief Print string to debug output
          *
-         * If there is already something on the output, puts space before
+         * If there is already something in the output, puts a space before
          * the value, unless @ref nospace() was set immediately before.
          * @see @ref operator<<(Debug&, const T&)
          */
@@ -438,18 +411,25 @@ class CORRADE_UTILITY_EXPORT Debug {
         /**
          * @brief Print UTF-32 character to debug output
          *
-         * Prints value as Unicode codepoint, i.e. @cb{.shell-session} U+0061 @ce.
+         * Prints the value as Unicode codepoint, i.e. @cb{.shell-session} U+0061 @ce.
          */
         Debug& operator<<(char32_t value);
 
         /**
          * @brief Print UTF-32 character literal to debug output
          *
-         * Prints value as list of Unicode codepoints, i.e.
-         * @cb{.shell-session} [U+0061, U+0062, U+0063} @ce.
+         * Prints the value as list of Unicode codepoints, i.e.
+         * @cb{.shell-session} {U+0061, U+0062, U+0063} @ce.
          */
         Debug& operator<<(const char32_t* value);
 
+        /**
+         * @brief Print a nullptr to debug output
+         *
+         * Prints the value as @cb{.shell-session} nullptr @ce.
+         */
+        Debug& operator<<(std::nullptr_t);
+
         #ifndef DOXYGEN_GENERATING_OUTPUT
         Debug& operator<<(Implementation::DebugOstreamFallback&& value);
         #endif
@@ -598,7 +578,7 @@ template<class ...Args> Debug& operator<<(Debug& debug, const std::tuple<Args...
  * @overload
  */
 template<class T, class U> Debug& operator<<(Debug& debug, const std::pair<T, U>& value) {
-    return debug << std::tuple<T, U>(value);
+    return debug << "(" << Debug::nospace << value.first << Debug::nospace << "," << value.second << Debug::nospace << ")";
 }
 
 /**
@@ -610,30 +590,6 @@ outputs.
 */
 class CORRADE_UTILITY_EXPORT Warning: public Debug {
     public:
-        #ifdef CORRADE_BUILD_DEPRECATED
-        /**
-         * @brief Warning output without newline at the end
-         * @deprecated Use @ref Warning(Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Warning(Flags) instead") static Warning noNewlineAtTheEnd() {
-            return Warning{Flag::NoNewlineAtTheEnd};
-        }
-
-        /**
-         * @brief Warning output without newline at the end
-         * @deprecated Use @ref Warning(std::ostream*, Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Warning(std::ostream*, Flags) instead") static Warning noNewlineAtTheEnd(std::ostream* output) {
-            return Warning{output, Flag::NoNewlineAtTheEnd};
-        }
-
-        /**
-         * @brief Set output for instances in this scope
-         * @deprecated Use @ref Warning(std::ostream*, Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Warning(std::ostream*, Flags) instead") static void setOutput(std::ostream* output);
-        #endif
-
         /**
          * @brief Current warning output stream
          *
@@ -709,30 +665,6 @@ class CORRADE_UTILITY_EXPORT Error: public Debug {
     friend Fatal;
 
     public:
-        #ifdef CORRADE_BUILD_DEPRECATED
-        /**
-         * @brief Error output without newline at the end
-         * @deprecated Use @ref Error(Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Error(Flags) instead") static Error noNewlineAtTheEnd() {
-            return Error{Flag::NoNewlineAtTheEnd};
-        }
-
-        /**
-         * @brief Error output without newline at the end
-         * @deprecated Use @ref Error(std::ostream*, Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Error(std::ostream*, Flags) instead") static Error noNewlineAtTheEnd(std::ostream* output) {
-            return Error{output, Flag::NoNewlineAtTheEnd};
-        }
-
-        /**
-         * @brief Set output for instances in this scope
-         * @deprecated Use @ref Error(std::ostream*, Flags) instead.
-         */
-        CORRADE_DEPRECATED("use Error(std::ostream*, Flags) instead") static void setOutput(std::ostream* output);
-        #endif
-
         /**
          * @brief Current error output stream
          *
@@ -856,14 +788,10 @@ class CORRADE_UTILITY_EXPORT Fatal: public Error {
         ~Fatal();
 
     private:
-        #ifdef CORRADE_BUILD_DEPRECATED
-        using Error::noNewlineAtTheEnd;
-        #endif
-
         int _exitCode;
 };
 
-/** @debugoperatorenum{Corrade::Utility::Debug::Color} */
+/** @debugoperatorclassenum{Debug,Debug::Color} */
 CORRADE_UTILITY_EXPORT Debug& operator<<(Debug& debug, Debug::Color value);
 
 namespace Implementation {
